global function speedometerPrecache
global function speedometerSettings

void function speedometerPrecache(){
    #if CLIENT
    thread speedometerInit()
    #endif
}

void function speedometerSettings(){
    #if UI
        #if S2_SPEED_HAS_MOD_SETTINGS
            AddModTitle("S2.Speedometer")
            AddModCategory("Main")
            AddConVarSettingEnum("s2_speedometer_metric", "Units", ["Imperial", "Metric"])
            AddConVarSettingEnum("s2_speedometer_hammer", "Display Hammer Units /s", ["Off", "On"])
            AddConVarSettingEnum("s2_speedometer_vertical", "Include Vertical Velocity", ["Off", "On"])
            AddConVarSettingEnum("s2_speedometer_fade", "Fade On Low Speed", ["Off", "On"])
            AddModCategory("UI")
            AddConVarSetting("s2_speedometer_alpha", "Alpha", "float")
            AddConVarSetting("s2_speedometer_size", "Size", "float")
            AddConVarSetting("s2_speedometer_pos", "Position", "vector")
            AddConVarSetting("s2_speedometer_col", "Colour", "vector")
        #else
            thread thread noSettingsDiagThread()
        #endif
    #endif
}

#if UI
void function noSettingsDiagThread(){
    WaitFrame()
    noSettingsDiag()
}

void function noSettingsDiag(){
    DialogData errorPopUp
    errorPopUp.header = "MISSING REQUIRED DEPENDENCY"
    errorPopUp.message = "SpeedometerV2 requires ModSettings to function.\nThe mod was not found; please download it."
    errorPopUp.image = $"ui/menu/common/dialog_error"
    AddDialogButton(errorPopUp, "Exit", void function(){ ClientCommand("exit") })
    AddDialogButton(errorPopUp, "Download", void function(){
        LaunchExternalWebBrowser("https://northstar.thunderstore.io/package/EladNLG/ModSettings/", WEBBROWSER_FLAG_FORCEEXTERNAL)
        noSettingsDiag()
    })
    OpenDialog(errorPopUp)
}
#endif

#if CLIENT
var speedRUI
bool usesMetric
bool addHammer
bool includeVertical
bool speedFade
vector position
vector colour
float alpha
float size

vector function GetConVarFloat3(string convar){
    array<string> value = split(GetConVarString(convar), " ")
    try{
        return Vector(value[0].tofloat(), value[1].tofloat(), value[2].tofloat()) 
    }
    catch(ex){
        throw "Invalid convar " + convar + "! make sure it is a float3 and formatted as \"X Y Z\""
    }
    unreachable
}

void function speedometerInit(){
    WaitFrame()
    speedRUI = RuiCreate( $"ui/cockpit_console_text_top_left.rpak", clGlobal.topoCockpitHudPermanent, RUI_DRAW_COCKPIT, -1 )
	RuiSetInt(speedRUI, "maxLines", 1)
	RuiSetInt(speedRUI, "lineNum", 1)
	RuiSetFloat2(speedRUI, "msgPos", <0.0, 0.0, 0.0>)
	RuiSetString(speedRUI, "msgText", "ERROR :(")
	RuiSetFloat(speedRUI, "msgFontSize", 0.0)
	RuiSetFloat(speedRUI, "msgAlpha", 0.0)
	RuiSetFloat(speedRUI, "thicken", 0.0)
	RuiSetFloat3(speedRUI, "msgColor", <0.0, 0.0, 0.0>)
    thread speedometerMain()
}

void function speedometerMain(){
    while(true){
        WaitFrame()
        if(!IsLobby()){
            entity player = GetLocalViewPlayer()
            if (player == null || !IsValid(player))
			    continue
            usesMetric = (GetConVarInt("s2_speedometer_metric") == 1)
            addHammer = (GetConVarInt("s2_speedometer_hammer") == 1)
            includeVertical = (GetConVarInt("s2_speedometer_vertical") == 1)
            speedFade = (GetConVarInt("s2_speedometer_fade") == 1)
            position = GetConVarFloat3("s2_speedometer_pos")
            colour = GetConVarFloat3("s2_speedometer_col")
            alpha = GetConVarFloat("s2_speedometer_alpha")
            size = GetConVarFloat("s2_speedometer_size")
            vector playerVelV = player.GetVelocity()
            float playerVel
            if(includeVertical)
                playerVel = sqrt(playerVelV.x * playerVelV.x + playerVelV.y * playerVelV.y + playerVelV.z * playerVelV.z)
            else
                playerVel = sqrt(playerVelV.x * playerVelV.x + playerVelV.y * playerVelV.y)
            float playerVelNormal = playerVel * (0.091392)
            if(!usesMetric)
                playerVelNormal *= (0.621371)
            string playerVelStr = format("%3i", playerVelNormal)
            if(usesMetric)
                playerVelStr += "kph"
            else
                playerVelStr += "mph"
            if(addHammer)
                playerVelStr += ("\n" + format("%3i", playerVel) + "HPS")
            if(playerVel/300 < alpha && speedFade)
                RuiSetFloat(speedRUI, "msgAlpha", (playerVel/300))
            else
                RuiSetFloat(speedRUI, "msgAlpha", alpha)
            RuiSetString(speedRUI, "msgText", playerVelStr)
            RuiSetFloat2(speedRUI, "msgPos", position)
            RuiSetFloat(speedRUI, "msgFontSize", size)
            RuiSetFloat3(speedRUI, "msgColor", colour)
        }
        else{
            RuiSetFloat(speedRUI, "msgAlpha", 0.0)
        }
    }
}
#endif